#include "shufflelist.h"
#include <iostream>

#ifndef max
#define   max(a, b)               ((a) < (b) ? (b) : (a))
#endif

using namespace std;

ShuffleList::ShuffleList (unsigned pWordSize, unsigned pWordCount)
{
  WordSize = pWordSize;
  WordCount = pWordCount;
  ElementCount = WordCount * WordSize;
  VariationCount = 1;
  for (unsigned i = WordSize; i > 0; i --)
    VariationCount *= ElementCount;

  Source1 = new unsigned[VariationCount];
  Source2 = new unsigned[VariationCount];
  Operation = new unsigned[VariationCount];
  Depth = new unsigned[VariationCount];
  for (unsigned i = 0; i < VariationCount; i ++)
    Depth[i] = 9999;

  Elements = new unsigned[VariationCount * WordSize];
  for (unsigned i = 0; i < VariationCount; i ++)
      Index2Variation (i, &Elements[i * WordSize]);

  PrimitiveCount = 0;
  Primitive = new unsigned[VariationCount];

  unsigned *V = new unsigned[WordSize];
  for (unsigned w = 0; w < WordCount; w ++)
  {
    for (unsigned e = 0; e < WordSize; e ++)
      V[e] = w * WordSize + e;
    unsigned VI = Variation2Index (V);
    Depth[VI] = 0;
  }
  delete[] V;

  ReV = new unsigned[WordSize];
}

ShuffleList::~ShuffleList ()
{
  delete[] Source1;
  delete[] Source2;
  delete[] Operation;
  delete[] Depth;
  delete[] Primitive;
  delete[] Elements;
  delete[] ReV;
}

unsigned ShuffleList::Variation2Index (unsigned *Variation)
{
  unsigned VI = 0;
  unsigned PI = 1;
  for (unsigned i = 0; i < WordSize; i ++)
    {
      VI += PI * Variation[i];
      PI *= ElementCount;
    };
  return VI;
}

void ShuffleList::Index2Variation (unsigned Index, unsigned *Variation)
{
  for (unsigned i = 0; i < WordSize; i ++)
    {
      Variation[i] = Index % ElementCount;
      Index /= ElementCount;
    }
}

void ShuffleList::AddOperation (unsigned *Variation)
{
  unsigned *S1 = new unsigned[WordSize];
  unsigned *S2 = new unsigned[WordSize];
  for (unsigned e = 0; e < WordSize; e ++)
  {
    S1[e] = e;
    S2[e] = WordSize + e;
  }

  unsigned VI = Variation2Index (Variation);
  if (Depth[VI] == 9999)
  {
    Source1[VI] = Variation2Index (S1);
    Source2[VI] = Variation2Index (S2);
    Depth[VI] = 1;
    Operation[VI] = VI;

    Primitive[PrimitiveCount] = VI;
    PrimitiveCount ++;
  }

  delete[] S1;
  delete[] S2;
}

void ShuffleList::Print ()
{
  unsigned *V = new unsigned[WordSize];
  for (unsigned i = 0; i < VariationCount; i ++)
    {
      if (Depth[i] == 9999)
	cout << "-: ";
      else
	cout << Depth[i] << ": ";
      PrintVariation (i);
      switch (Depth[i])
	{
	case 9999:
	  cout << "<--???\n";
	  break;
	case 0:
	  cout << endl;
	  break;
	default:
	  cout << "<--";
	  PrintVariation (Operation[i]);
	  cout << "--";
	  PrintVariation (Source1[i]);
	  cout << ',';
	  PrintVariation (Source2[i]);
	  cout << endl;
	  break;
	}
    }
}

void ShuffleList::PrintVariation (unsigned VI)
{
  unsigned *V = &Elements[WordSize * VI];
  cout << '(';
  for (unsigned i = 0; i < WordSize; i ++)
    {
      if (i != 0) cout << ',';
      cout << V[i];
    }
  cout << ')';
}

void ShuffleList::PrintOperation (unsigned O)
{
  PrintVariation (O);
}

unsigned ShuffleList::Apply (unsigned Op, unsigned S1, unsigned S2)
{
  unsigned *OpV = &Elements[Op * WordSize];
  unsigned *S1V = &Elements[S1 * WordSize];
  unsigned *S2V = &Elements[S2 * WordSize];

  for (unsigned i = 0; i < WordSize; i ++)
  {
    unsigned si = OpV[i];
    if   (si < WordSize)
         ReV[i] = S1V[si];
    else ReV[i] = S2V[si - WordSize];
  }

  unsigned Re = Variation2Index (ReV);

  return Re;
}

void ShuffleList::Calculate ()
{
  unsigned NewDepth = 1;
  bool Changed = true;
  unsigned FoundCount = PrimitiveCount + WordCount;
  while (Changed && FoundCount < VariationCount)
  {
      Changed = false;
      for (unsigned p = 0; p < PrimitiveCount; p ++)
      {
	  unsigned Prim = Primitive[p];
	  for (unsigned s1 = 0; s1 < VariationCount && FoundCount < VariationCount; s1 ++)
	    if (Depth[s1] < NewDepth)
	      for (unsigned s2 = 0; s2 < VariationCount; s2 ++)
                if (Depth[s1] + Depth[s2] + 1 == NewDepth)
                {
                  unsigned Re = Apply (Prim, s1, s2);
	          if (Depth[Re] == 9999)
	          {
                    Depth[Re] = Depth[s1] + Depth[s2] + 1;
                    Operation[Re] = Prim;
                    Source1[Re] = s1;
                    Source2[Re] = s2;
                    Changed = true;
                    FoundCount ++;
                    cout << "found " << FoundCount << " of " << VariationCount << "\r";
                  }
		}
      }
      NewDepth ++;
  }
  cout << endl;
}

