#include "shufflesequence.h"
#include <algorithm>
#include <iostream>
#include <functional>

class DepthOrder : public binary_function<unsigned,unsigned,bool>
{
public:
  DepthOrder (ShuffleList *pSL) { SL = pSL; }
  bool operator() (unsigned a, unsigned b)
  { unsigned da = SL->GetDepth (a);
    unsigned db = SL->GetDepth (b);
    return   da < db || (da == db && a < b);
  }
  ShuffleList *SL;
};

ShuffleSequence::ShuffleSequence (ShuffleList *pSL, unsigned Target)
{
  SL = pSL;
  AddResult (Target);
  sort (Results.begin (), Results.end (), DepthOrder (SL));
}

ShuffleSequence::ShuffleSequence (ShuffleList *pSL, unsigned *Target)
{
  SL = pSL;
  AddResult (pSL->Variation2Index (Target));
  sort (Results.begin (), Results.end (), DepthOrder (SL));
}

void ShuffleSequence::AddResult (unsigned V)
{
  if (find (Results.begin (), Results.end (), V) == Results.end ())
  {
    Results.push_back (V);
    if (SL->GetDepth (V) > 0)
    {
      AddResult (SL->GetArg1 (V));
      AddResult (SL->GetArg2 (V));
    }
  }
}

void ShuffleSequence::Print (void (&PrintOperation) (unsigned O, ShuffleList *SL))
{
  for (unsigned i = 0; i < Results.size (); i ++)
  {
    unsigned r = Results[i];
    if (SL->GetDepth (r) == 0)
    {
      cout << (char) ('a' + i) << " = ";
      SL->PrintVariation (r);
      cout << endl;
    }
    else
    {
      cout << (char) ('a' + i) << " = ";
      PrintOperation (SL->GetOpOfVar (r), SL);
      cout << " (" << GetResultName (SL->GetArg1 (r))
           << ", " << GetResultName (SL->GetArg2 (r))
           << ") = ";
      SL->PrintVariation (r);
      cout << endl;
    }
  }
}

char ShuffleSequence::GetResultName (unsigned V)
{
  unsigned i = 0;
  while (Results[i] != V) i ++;
  return 'a' + i;
}
