#include "shufflesequence.h"
#include <iostream>

using namespace std;

void PrintOperation (unsigned O, ShuffleList *SL)
{
  unsigned *OV = new unsigned[4];
  SL->Index2Variation (O, OV);
  if (OV[1] < 4)
     cout << "shufps["
          << OV[0] << ',' << OV[1] << ',' 
          << (OV[2] - 4) << ',' << (OV[3] - 4)
          << ']';
  else if (OV[0] == 0)
     cout << "unpcklps";
  else if (OV[0] == 2)
     cout << "unpckhps";
  else
     cout << "unknown["
          << OV[0] << ',' << OV[1] << ',' 
          << OV[2] << ',' << OV[3]
          << ']';
  delete[] OV;
}

int main (int argc, char *argv[])
{
  if (argc != 6)
  {
     cerr << "usage: sseshuffler wordcount e0 e1 e2 e3\n\n"
          << "  wordcount   ... number of packed input words containing 0,1,2,3; 4,5,6,7; ...\n"
          << "  e0,e1,e2,e3 ... the target word\n\n"
          << "example:\n\n"
          << "  sseshuffler 2 1 2 3 4\n\n"
          << "  shows the commands to get the word 1,2,3,4 from the two words 0,1,2,3; 4,5,6,7\n";
     exit (0);
  }

  ShuffleList SL (4, max (atoi (argv[1]), 2));
  unsigned O[4];

  O[0] = 0; O[1] = 4; O[2] = 1; O[3] = 5;
  SL.AddOperation (O);

  O[0] = 2; O[1] = 6; O[2] = 3; O[3] = 7;
  SL.AddOperation (O);

  for (unsigned s0 = 0; s0 < 4; s0 ++)
  for (unsigned s1 = 0; s1 < 4; s1 ++)
  for (unsigned s2 = 4; s2 < 8; s2 ++)
  for (unsigned s3 = 4; s3 < 8; s3 ++)
  {
    O[0] = s0; O[1] = s1; O[2] = s2; O[3] = s3;
    SL.AddOperation (O);
  }

  SL.Calculate ();

  O[0] = atoi (argv[2]);
  O[1] = atoi (argv[3]);
  O[2] = atoi (argv[4]);
  O[3] = atoi (argv[5]);
  ShuffleSequence SS (&SL, O);
  SS.Print (PrintOperation);
}

